import os
import numpy as np
from modules import ClassificationModule
from torch.utils.data import DataLoader
import torch
from dataset_Bfull_mhi import SLRDataset as ds_mhi
from dataset_Bfull import SLRDataset as ds_rgb
from tqdm import tqdm
import functools
import pickle
import torch.multiprocessing

torch.multiprocessing.set_sharing_strategy('file_system')

def disk_cache(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        cache_dir = 'cache/bospfull'
        os.makedirs(cache_dir, exist_ok=True)
        args_path = str(args[0]).replace('/','_')
        cache_file = os.path.join(cache_dir, f'{func.__name__}_{args_path}_{kwargs}.cache')
        
        try:
            with open(cache_file, 'rb') as f:
                return pickle.load(f)
        except (FileNotFoundError, pickle.UnpicklingError):
            result = func(*args, **kwargs)
            with open(cache_file, 'wb') as f:
                pickle.dump(result, f)
            return result

    return wrapper


def test_SVM(train, train_label, test, test_label):
    from sklearn.svm import SVC
    svc = SVC(random_state=60)
    svc.fit(train, train_label)
    scr = svc.score(test, test_label)
    return scr
    
def test(model,ds):
    model.cuda()
    model.eval()
    feature_list = []
    label_list = []
    bs = 32
    dl = DataLoader(ds,bs,num_workers=4)
    for idx, (data, label) in enumerate(tqdm(dl)):
        with torch.no_grad():
            res = model.model.stem(data.cuda())
            res = model.model.model(res.cuda()).cpu()
            feature_list.extend(res.reshape(-1,512).numpy())
        label_list.extend(label)
        
    feature_list = np.asarray(feature_list,dtype=np.float32)
        
    return feature_list, label_list

device = torch.device("cuda:0") if torch.cuda.is_available() else torch.device("cpu")

def get_feature(save_name, modelName = 'resnet18v6',  dataModule = None, data_name = None, stem=0, act = None):
    if 'mergehands' in save_name:
        size = (56,112)
    else:
        size = (112,112)
    data_name = data_name + ".hdf5"
    ds_train = dataModule(phase='train',dataset_name=data_name, evaluate=True,size=size)
    ds_val = dataModule(phase='val', dataset_name=data_name, evaluate=True,size=size)
    ds_test = dataModule(phase='test', dataset_name=data_name, evaluate=True,size=size)
    
    first_filter_size = ds_train[0][0].shape[-3]
    print("first_filter_size", first_filter_size)
    
    module_setting = {
                      'model_name' : modelName,
                      'model_hparams':{"num_classes": 744,
                                       "pretrained" : True,
                                        "first_filter_size":first_filter_size,
                                        "act":act,
                                     },
                      'optimizer_name':"SGD",
                      'optimizer_hparams':{"lr": 1e-3,
                                            "momentum":0.9,
                                          },
                      'lr_scheduler_hparams':{
                          'step_size': 15
                      }
    }
    
    model = ClassificationModule(**module_setting)
    model = model.load_from_checkpoint(save_name)
    
    X_train, y_train = test(model, ds_train)
    X_test, y_test = test(model, ds_test)
    
    scr = test_SVM(X_train, y_train, X_test, y_test)
    
    return X_train, y_train, X_test, y_test, scr


def fusion(all_data):
    train_data = []
    test_data = []
    train_label = []
    test_label = []
    
    for X_train, y_train, X_test, y_test, scr in all_data:
        train_data.append(X_train)
        test_data.append(X_test)
        train_label = y_train
        test_label = y_test
        print(scr)
    
    train_data = np.concatenate(train_data, axis=1)
    test_data = np.concatenate(test_data, axis=1)
    
    return train_data, train_label, test_data, test_label


all_data = []

@disk_cache
def get_final_features(save_name):
    save_name = getCKPT(save_name)
    modelName = save_name.split('/')[4].split('_')[0]
    dataName = save_name.split('/')[4].split('_')[1]
    stem = save_name.split('/')[4].split('_')[-1]
    act = save_name.split('/')[3]
    if 'MHI' in save_name.split('/')[1]:
        ds = ds_mhi
        if dataName != 'finger':
            dataName = dataName + '_mhi_'+save_name.split('/')[4].split('_')[3]
        else:
            dataName = dataName + '_mhi'
    else:
        ds = ds_rgb
    
    tempData = get_feature(save_name,  modelName = modelName,  dataModule = ds, data_name= dataName, stem = stem, act = act)
    return tempData

import glob

def getCKPT(path):
    return glob.glob("saved_model/"+path +"/version_*/*/*.ckpt")[0]


ll = ['BODY','HANDS','MERGEHANDS','FACE',
      'BODY_MHI','HANDS_MHI','FINGER_MHI','FACE_MHI']


BODY = get_final_features('.../body')
HANDS = get_final_features('.../hands')
MERGEHANDS = get_final_features('.../mergehands')
FACE = get_final_features('.../face')

BODY_MHI = get_final_features('.../body_mhi_3')
HANDS_MHI = get_final_features('.../hands_mhi_3')
FINGER_MHI = get_final_features('.../finger_mhi')
FACE_MHI = get_final_features('.../face_mhi_3')

all_data = [BODY, BODY_MHI, HANDS,HANDS_MHI, FINGER_MHI, FACE, FACE_MHI, MERGEHANDS]
train_data, train_label, test_data, test_label = fusion(all_data)
result = test_SVM(train_data, train_label, test_data, test_label)
print(result)