import torch
import torchvision
import torch.nn as nn

def convert_relu_to_rrelu(model, act):
    """
    Convert all ReLU activations in the model to RReLU or another specified activation.
    :param model: PyTorch model to convert activations in.
    :param act: String name of the activation function to use (must be an attribute of torch.nn).
    """
    act_fun = getattr(nn, act)()
    for child_name, child in model.named_children():
        if isinstance(child, nn.ReLU):
            setattr(model, child_name, act_fun)
        else:
            convert_relu_to_rrelu(child, act)

class Resnet18(torch.nn.Module):
    def __init__(self, pretrained=False, num_classes=None, first_filter_size=1, stem=1, act=''):
        super().__init__()
        self.stem = nn.Conv2d(first_filter_size, 3, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)
        self.model = torchvision.models.resnet18(pretrained=pretrained)
        # Convert activations in stem and main model if an activation is specified.
        if act:
            convert_relu_to_rrelu(self.stem, act=act)
            convert_relu_to_rrelu(self.model, act=act)
        self.model.fc = nn.Identity()
        self.fc1 = torch.nn.Linear(512, num_classes)
        self.dropout = torch.nn.Dropout(p=0.4)

    def forward(self, x):
        x = self.stem(x)
        x = self.model(x)
        x = self.dropout(x)
        x = self.fc1(x)
        return x
