import pytorch_lightning as pl
import torch.nn as nn
import torch.optim as optim
import importlib.util
from utils import list_classes_from_module

def load_model(model_name):
    """Attempts to load a model class from a given module name.
    
    Args:
        model_name (str): The name of the model to load.
        
    Returns:
        The last class found in the module if successful, None otherwise.
    """
    try:
        module = importlib.import_module(model_name)
        model_class_definitions = list_classes_from_module(module)
        return model_class_definitions[-1]
    except ModuleNotFoundError:
        return None

def create_model(model_name, model_hparams):
    """Creates a model instance from a model name and hyperparameters.
    
    Args:
        model_name (str): The model's module name.
        model_hparams (dict): Hyperparameters for the model.
        
    Returns:
        An instance of the model.
        
    Raises:
        ValueError: If the model name is unknown.
    """
    model = load_model("models." + model_name) or load_model(model_name)
    if model:
        return model(**model_hparams)
    raise ValueError(f"Unknown model name \"{model_name}\".")

class ClassificationModule(pl.LightningModule):
    """A PyTorch Lightning module for image classification."""
    
    def __init__(self, model_name, model_hparams, optimizer_name, optimizer_hparams, lr_scheduler_hparams):
        super().__init__()
        self.save_hyperparameters()
        self.model = create_model(model_name, model_hparams)
        self.loss_module = nn.CrossEntropyLoss()

    def forward(self, imgs):
        """Defines the forward pass."""
        return self.model(imgs)

    def configure_optimizers(self):
        """Sets up the optimizer and learning rate scheduler."""
        optimizers = {
            "Adam": optim.AdamW(self.parameters(), **self.hparams.optimizer_hparams),
            "SGD": optim.SGD(self.parameters(), **self.hparams.optimizer_hparams)
        }
        
        optimizer = optimizers.get(self.hparams.optimizer_name)
        if not optimizer:
            raise ValueError(f"Unknown optimizer: \"{self.hparams.optimizer_name}\"")
        
        scheduler = optim.lr_scheduler.StepLR(optimizer, step_size=self.hparams.lr_scheduler_hparams['step_size'], gamma=0.1)
        return [optimizer], [{'scheduler': scheduler}]

    def training_step(self, batch, batch_idx):
        imgs, labels = batch
        preds = self.model(imgs)
        loss = self.loss_module(preds, labels)
        acc = (preds.argmax(dim=-1) == labels).float().mean()
        self.log('train_acc', acc, on_step=False, on_epoch=True)
        self.log('train_loss', loss)
        return loss

    def validation_step(self, batch, batch_idx):
        imgs, labels = batch
        preds = self.model(imgs).argmax(dim=-1)
        acc = (labels == preds).float().mean()
        self.log('val_acc', acc)

    def test_step(self, batch, batch_idx):
        imgs, labels = batch
        preds = self.model(imgs).argmax(dim=-1)
        acc = (labels == preds).float().mean()
        self.log('test_acc', acc)
