import pytorch_lightning as pl
import os
from pytorch_lightning.callbacks import LearningRateMonitor, ModelCheckpoint, TQDMProgressBar
import torch
from pathlib import Path
from pytorch_lightning.loggers import TensorBoardLogger
import json
import numpy as np
import importlib.util
import shutil

CHECKPOINT_PATH = '.'
device = torch.device("cuda:0") if torch.cuda.is_available() else torch.device("cpu")

def train_model(module, train_loader, val_loader=None, test_loader=None, save_name=None, pretrained_model = False, checkpoint_path = None, transfer_learning=False, **kwargs):
    """
    Inputs:
        model_name - Name of the model you want to run. Is used to look up the class in "model_dict"
        save_name (optional) - If specified, this name will be used for creating the checkpoint and logging directory.
    """
    if save_name is None:
        save_name = module.model_name
        
    trainer = pl.Trainer(default_root_dir=os.path.join(CHECKPOINT_PATH, save_name),
                         gpus=1 if str(device)=="cuda:0" else 0,
                         max_epochs=kwargs['max_epochs'],
                         callbacks=[ModelCheckpoint(save_weights_only=False, mode="max", monitor=kwargs['monitor']),
                                    LearningRateMonitor("epoch")],
                         progress_bar_refresh_rate=1,
                         logger=TensorBoardLogger(save_dir="saved_models/",name=save_name))

    trainer.logger._log_graph = True
    trainer.logger._default_hp_metric = None

    # pretrained model loader
    pretrained_filename = ""
    max_result = -1
    if pretrained_model:
        val_result_list = []
        checkpoint_list = []

        for res in sorted(Path(trainer.logger.log_dir).parent.glob("*/result.json")):
            with open(str(res), 'r') as f:
                loaded_result = json.load(f)
            val_result_list.append(loaded_result['val'])
            checkpoint_list.append(res.parent)
        
        if checkpoint_path is not None:
            pretrained_filename = list(Path(checkpoint_path).glob('*/*.ckpt'))[0]
        else:
            if len(val_result_list) != 0:
                sel_idx = np.argmax(val_result_list)
                max_result = np.max(val_result_list)
                pretrained_filename = list(checkpoint_list[sel_idx].glob('*/*.ckpt'))[0]
            else:
                pass

    if os.path.isfile(pretrained_filename) and not transfer_learning:
        print(f"Found pretrained model at {pretrained_filename}, loading...")

        trainer = pl.Trainer(default_root_dir=os.path.join(CHECKPOINT_PATH, save_name),
                             gpus=1 if str(device)=="cuda:0" else 0,
                             gpus=kwargs['gpus'],
                             max_epochs=kwargs['max_epochs'],
                             callbacks=[ModelCheckpoint(save_weights_only=False, mode=kwargs['mode'], monitor=kwargs['monitor']),
                                        LearningRateMonitor("epoch")],
                             progress_bar_refresh_rate=1,
                             logger=TensorBoardLogger(save_dir="saved_models/",name=save_name),
                             resume_from_checkpoint = pretrained_filename
                             )
        
        pl.seed_everything(42)
        if val_loader is not None:
            trainer.fit(module, train_loader, val_loader)
        else:
            trainer.fit(module, train_loader)

        model = module.load_from_checkpoint(trainer.checkpoint_callback.best_model_path) # Load best checkpoint after training
        
        
    elif os.path.isfile(pretrained_filename) and transfer_learning:
        print(f"Found pretrained model at {pretrained_filename}, loading...")
        print("Transfer Learning")
        model = module.load_from_checkpoint(pretrained_filename)
        model.model.fc.out_features = 226
        
        trainer = pl.Trainer(default_root_dir=os.path.join(CHECKPOINT_PATH, save_name),
                             gpus=1 if str(device)=="cuda:0" else 0,
                             max_epochs=kwargs['max_epochs'],
                             callbacks=[ModelCheckpoint(save_weights_only=False, mode="max", monitor=kwargs['monitor']),
                                        LearningRateMonitor("epoch")],
                             progress_bar_refresh_rate=1,
                             logger=TensorBoardLogger(save_dir="saved_models/",name=save_name),
                             # resume_from_checkpoint = pretrained_filename
                             )
        if val_loader is not None:
            trainer.fit(model, train_loader, val_loader)
        else:
            trainer.fit(model, train_loader)

        model = model.load_from_checkpoint(trainer.checkpoint_callback.best_model_path) # Load best checkpoint after training
        
    else:
        pl.seed_everything(42)
        if val_loader is not None:
            trainer.fit(module, train_loader, val_loader)
        else:
            trainer.fit(module, train_loader)

        model = module.load_from_checkpoint(trainer.checkpoint_callback.best_model_path) # Load best checkpoint after training

    # Test best model on validation and test set
    
    result = {}
    if val_loader is not None:
        val_result = trainer.test(model, val_loader, verbose=False)
        result.update({"val": val_result[0][kwargs['evaluate_metric']]})
    if test_loader is not None:
        test_result = trainer.test(model, test_loader, verbose=False)
        result.update({"test": test_result[0][kwargs['evaluate_metric']]})
    
    results_path = trainer.logger.log_dir + '/result.json'

    with open(results_path, 'a') as f:
        json.dump(result, f)
        
    if max_result != -1:
        result.update({"best_val_prev_value":max_result})
    
    result.update({"checkpoint_path":trainer.checkpoint_callback.best_model_path})
    result.update({"model_path":importlib.import_module(module.model.__module__).__file__})
    
    submodule = importlib.import_module(model.__module__)
    result.update({"module_path":submodule.__file__})
    
    submoduleDS = importlib.import_module(train_loader.dataset.__module__)
    result.update({"dataset_path":submoduleDS.__file__})
    
    shutil.copy(submodule.__file__, str(Path(trainer.checkpoint_callback.dirpath).parent))
    shutil.copytree("models", str(Path(trainer.checkpoint_callback.dirpath).parent/"models"))
    
    results_path = trainer.logger.log_dir + '/result2.json'
    with open(results_path, 'a') as f:
        json.dump(result, f)
    
    return module, result
