import os
from modules import ClassificationModule
from torch.utils.data import DataLoader
import pytorch_lightning as pl
import torch
from pytorch_lightning.callbacks import LearningRateMonitor, ModelCheckpoint
from pytorch_lightning.loggers import TensorBoardLogger
from pathlib import Path
import json

def json_add(file, data):
    try:
        with open(file) as json_file:
            curr_data = json.load(json_file)
            
        curr_data.update(data)
        with open(file, 'w') as outfile:
            json.dump(curr_data, outfile)
    except:
        with open(file, 'w') as outfile:
            json.dump(data, outfile)

CHECKPOINT_PATH = '.'
device = torch.device("cuda:0") if torch.cuda.is_available() else torch.device("cpu")


def params2str(params):
    str_params = params['project_name'] + '_' + params['completed'] + '/act/' + params['act'] + '/' + params['model_name'] + '_' + params['dataset_name']
    return str_params

def train(params, ds, ds_mhi):    
    save_root = 'saved_model/'
    save_name = params2str(params)
    data_name = params['dataset_name'] + ".hdf5"
    comp = params['completed']
    model_name = params['model_name']
    size = params['size']
    bs = params['bs']
    num_classes = params['num_classes']
    act = params['act']
    
    exp = None
    try:
        exp = params['experiments']
        usr = params['usr']
    except Exception:
        pass
    
    if exp is None:
        if 'mhi' in data_name:
            ds_train = ds_mhi(phase='train', dataset_name=data_name, completed=comp)
            ds_val = ds_mhi(phase='val', dataset_name=data_name,completed=comp)
            ds_test = ds_mhi(phase='test', dataset_name=data_name,completed=comp)
        else:
            ds_train = ds(phase='train', dataset_name=data_name, completed=comp,size=size)
            ds_val = ds(phase='val', dataset_name=data_name,completed=comp,size=size)
            ds_test = ds(phase='test', dataset_name=data_name,completed=comp,size=size)
    else:
        if 'mhi' in data_name:
            ds_train = ds_mhi(phase='train', dataset_name=data_name, completed=comp,experiments=exp, usr=usr)
            ds_val = ds_mhi(phase='val', dataset_name=data_name,completed=comp,experiments=exp, usr=usr)
            ds_test = ds_mhi(phase='test', dataset_name=data_name,completed=comp,experiments=exp, usr=usr)
        else:
            ds_train = ds(phase='train', dataset_name=data_name, completed=comp,size=size,experiments=exp, usr=usr)
            ds_val = ds(phase='val', dataset_name=data_name,completed=comp,size=size,experiments=exp, usr=usr)
            ds_test = ds(phase='test', dataset_name=data_name,completed=comp,size=size,experiments=exp, usr=usr)
        
    first_filter_size = ds_train[0][0].shape[-3]
        
    print(f"Train :{len(ds_train)}")
    print(f"Val :{len(ds_val)}")
    print(f"Test :{len(ds_test)}")
    
    train_loader = DataLoader(ds_train, batch_size=bs, shuffle=True, drop_last=True, num_workers=6)
    val_loader = DataLoader(ds_val, batch_size=bs, shuffle=False, drop_last=True, num_workers=6)
    test_loader = DataLoader(ds_test, batch_size=bs, shuffle=False, drop_last=True, num_workers=6)
        
    trainer = pl.Trainer(default_root_dir=os.path.join(CHECKPOINT_PATH, save_name),
                         # gpus=1 if str(device)=="cuda:0" else 0,
                         gpus=[0],
                         # accelerator='dp',# We run on a single GPU (if possible)
                         max_epochs=35,
                         callbacks=[ModelCheckpoint(save_weights_only=False, mode="max", monitor='val_acc'),
                                    LearningRateMonitor("epoch")],
                         progress_bar_refresh_rate=1,
                         logger=TensorBoardLogger(save_dir=save_root,name=save_name),
                         )
    
    module_setting = {
                      'model_name' : model_name,
                      'model_hparams':{"num_classes": num_classes,
                                       "pretrained" : True,
                                        "first_filter_size":first_filter_size,
                                        "act":act,
                                     },
                      'optimizer_name':"SGD",
                      'optimizer_hparams':{"lr": 1e-3,
                                            "momentum":0.9,
                                          },
                      'lr_scheduler_hparams':{
                          'step_size': 15
                      }
    }
    
    model = ClassificationModule(**module_setting)
    print(model)
    
    trainer.fit(model, train_loader,val_loader)
    
    model = model.load_from_checkpoint(trainer.checkpoint_callback.best_model_path)

    test_acc=trainer.test(model, test_loader)
    
    
    pfile = Path(trainer.checkpoint_callback.best_model_path).parent
    result_data = pfile/ 'result_test.json'

    res = {save_name:test_acc}
    json_add(result_data, res)
    
    
    result_data = save_root+'/result_data.json'
    res = {save_name:test_acc}
    print(res)
    json_add(result_data, res)
    
    return model, trainer

