# Copyright 2021 The MediaPipe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless requi_RED by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""MediaPipe solution drawing styles."""

from typing import Mapping, Tuple
from mediapipe.python.solutions import pose_connections
from mediapipe.python.solutions import face_mesh_connections
from mediapipe.python.solutions import hands_connections
from mediapipe.python.solutions.drawing_utils import DrawingSpec
from mediapipe.python.solutions.hands import HandLandmark
from mediapipe.python.solutions.pose import PoseLandmark
import numpy as np


_RADIUS = 5
_RED = (48, 48, 255)
_GREEN = (48, 255, 48)
_BLUE = (192, 101, 21)
_YELLOW = (0, 204, 255)
_GRAY = (128, 128, 128)
_PURPLE = (128, 64, 128)
_PEACH = (180, 229, 255)
_WHITE = (224, 224, 224)
_BLACK = (0, 0, 0)


_ORANGE = (0,255, 165)
_PINK = (203, 255, 192)
_INDIANRED = (92, 205, 92)

_SEAGREEN = (87, 46, 139)
_PALEGREEN = (152, 152, 251)

_LIGHTBLUE = (230, 173, 216)
_ROYALBLUE = (225, 65, 105)

_GOLD = (0, 215, 255)
_GREENYELLOW = (47, 173, 255)


_VIOLET = (238, 238, 130)
_INDIGO = (130, 75, 0)

_PLUM = (221, 221, 160)
_BROWN = (96,244, 164)

_BLACK = (0,0, 0)

# renkler = np.random.randint(30, 225, (5, 3))


renkler =  [(200, 75, 30),
            (30, 60, 200),
            (60,  170,  220),
            (40, 200, 60),
            (230,  120,  190)]

# renkler =  [(255, 0, 0),
#             (0, 255, 0),
#             (0,  0,  255),
#             (0, 255, 0),
#             (0,  0,  255)]

# Hands
_THICKNESS_WRIST_MCP = 5
_THICKNESS_FINGER = 15
_THICKNESS_DOT = -1

# Hand landmarks
_PALM_LANMARKS = (HandLandmark.WRIST, HandLandmark.THUMB_CMC,
                  HandLandmark.INDEX_FINGER_MCP, HandLandmark.MIDDLE_FINGER_MCP,
                  HandLandmark.RING_FINGER_MCP, HandLandmark.PINKY_MCP)
_THUMP_LANDMARKS = (HandLandmark.THUMB_MCP, HandLandmark.THUMB_IP,
                    HandLandmark.THUMB_TIP)
_INDEX_FINGER_LANDMARKS = (HandLandmark.INDEX_FINGER_PIP,
                           HandLandmark.INDEX_FINGER_DIP,
                           HandLandmark.INDEX_FINGER_TIP)
_MIDDLE_FINGER_LANDMARKS = (HandLandmark.MIDDLE_FINGER_PIP,
                            HandLandmark.MIDDLE_FINGER_DIP,
                            HandLandmark.MIDDLE_FINGER_TIP)
_RING_FINGER_LANDMARKS = (HandLandmark.RING_FINGER_PIP,
                          HandLandmark.RING_FINGER_DIP,
                          HandLandmark.RING_FINGER_TIP)
_PINKY_FINGER_LANDMARKS = (HandLandmark.PINKY_PIP, HandLandmark.PINKY_DIP,
                           HandLandmark.PINKY_TIP)
_HAND_LANDMARK_STYLE = {
    _PALM_LANMARKS:
        DrawingSpec(
            color=_RED, thickness=_THICKNESS_DOT, circle_radius=_RADIUS),
    _THUMP_LANDMARKS:
        DrawingSpec(
            color=_PEACH, thickness=_THICKNESS_DOT, circle_radius=_RADIUS),
    _INDEX_FINGER_LANDMARKS:
        DrawingSpec(
            color=_PURPLE, thickness=_THICKNESS_DOT, circle_radius=_RADIUS),
    _MIDDLE_FINGER_LANDMARKS:
        DrawingSpec(
            color=_YELLOW, thickness=_THICKNESS_DOT, circle_radius=_RADIUS),
    _RING_FINGER_LANDMARKS:
        DrawingSpec(
            color=_GREEN, thickness=_THICKNESS_DOT, circle_radius=_RADIUS),
    _PINKY_FINGER_LANDMARKS:
        DrawingSpec(
            color=_BLUE, thickness=_THICKNESS_DOT, circle_radius=_RADIUS),
}

# Hands connections
_HAND_CONNECTION_STYLE = {
    hands_connections.HAND_PALM_CONNECTIONS:
        DrawingSpec(color=_GRAY, thickness=_THICKNESS_WRIST_MCP),
    hands_connections.HAND_THUMB_CONNECTIONS:
        DrawingSpec(color=renkler[0], thickness=_THICKNESS_FINGER),
    hands_connections.HAND_INDEX_FINGER_CONNECTIONS:
        DrawingSpec(color=renkler[1], thickness=_THICKNESS_FINGER),
    hands_connections.HAND_MIDDLE_FINGER_CONNECTIONS:
        DrawingSpec(color=renkler[2], thickness=_THICKNESS_FINGER),
    hands_connections.HAND_RING_FINGER_CONNECTIONS:
        DrawingSpec(color=renkler[3], thickness=_THICKNESS_FINGER),
    hands_connections.HAND_PINKY_FINGER_CONNECTIONS:
        DrawingSpec(color=renkler[4], thickness=_THICKNESS_FINGER)
}
    
    

_THICKNESS_FINGER_SINGLE = 4
    
_HAND_CONNECTION_STYLE_THUMB = {
    hands_connections.HAND_THUMB_CONNECTIONS:
        DrawingSpec(color=renkler[0], thickness=_THICKNESS_FINGER_SINGLE),
}
    
_HAND_CONNECTION_STYLE_INDEX = {
    hands_connections.HAND_INDEX_FINGER_CONNECTIONS:
        DrawingSpec(color=renkler[1], thickness=_THICKNESS_FINGER_SINGLE),
}

_HAND_CONNECTION_STYLE_MIDDLE = {
    hands_connections.HAND_MIDDLE_FINGER_CONNECTIONS:
        DrawingSpec(color=renkler[2], thickness=_THICKNESS_FINGER_SINGLE),
}
    
_HAND_CONNECTION_STYLE_RING = {
    hands_connections.HAND_RING_FINGER_CONNECTIONS:
        DrawingSpec(color=renkler[3], thickness=_THICKNESS_FINGER_SINGLE),
}

_HAND_CONNECTION_STYLE_PINKY = {
    hands_connections.HAND_PINKY_FINGER_CONNECTIONS:
        DrawingSpec(color=renkler[4], thickness=_THICKNESS_FINGER_SINGLE)
}
    
_HAND_CONNECTION_STYLE_ARRAYS = [_HAND_CONNECTION_STYLE_THUMB,
                                 _HAND_CONNECTION_STYLE_INDEX,
                                 _HAND_CONNECTION_STYLE_MIDDLE,
                                 _HAND_CONNECTION_STYLE_RING,
                                 _HAND_CONNECTION_STYLE_PINKY]
    
    
# Hands connections
# _HAND_CONNECTION_STYLE = {
#     hands_connections.HAND_PALM_CONNECTIONS:
#         DrawingSpec(color=_GRAY, thickness=_THICKNESS_WRIST_MCP),
#     hands_connections.HAND_THUMB_CONNECTIONS:
#         DrawingSpec(color=_GRAY, thickness=_THICKNESS_FINGER),
#     hands_connections.HAND_INDEX_FINGER_CONNECTIONS:
#         DrawingSpec(color=_GRAY, thickness=_THICKNESS_FINGER),
#     hands_connections.HAND_MIDDLE_FINGER_CONNECTIONS:
#         DrawingSpec(color=_GRAY, thickness=_THICKNESS_FINGER),
#     hands_connections.HAND_RING_FINGER_CONNECTIONS:
#         DrawingSpec(color=_GRAY, thickness=_THICKNESS_FINGER),
#     hands_connections.HAND_PINKY_FINGER_CONNECTIONS:
#         DrawingSpec(color=_GRAY, thickness=_THICKNESS_FINGER)
# }
        
    
# ((12,14),(14,16),(16,18),(16,20),(16,22))

_POSE_CONNECTION_STYLE = {
    pose_connections.POSE_CONNECTIONS:
        DrawingSpec(color=_PEACH, thickness=19)
}
    

thickness_POSE_CONNECTION_STYLE2 = 19
_POSE_CONNECTION_STYLE2 = {
    ((11, 12),(11, 23), (12, 24), (23, 24), (23, 25),(24, 26), (25, 27), (26, 28), (27, 29), (28, 30), (29, 31), (30, 32), (27, 31), (28, 32)):
        DrawingSpec(color=_PEACH, thickness=thickness_POSE_CONNECTION_STYLE2),
    ((0, 1), (1, 2), (2, 3), (3, 7), (0, 4), (4, 5), (5, 6), (6, 8), (9, 10)):
        DrawingSpec(color=_PEACH, thickness=thickness_POSE_CONNECTION_STYLE2),
    ((12,14),(14,16),(16,18),(16,20),(16,22),(18, 20)):
        DrawingSpec(color=_SEAGREEN, thickness=thickness_POSE_CONNECTION_STYLE2),
    ((11, 13),(13, 15), (15, 17), (15, 19), (15, 21), (17, 19)):
        DrawingSpec(color=_SEAGREEN, thickness=thickness_POSE_CONNECTION_STYLE2)
}

# FaceMesh connections
_THICKNESS_TESSELATION = 1
_THICKNESS_CONTOURS = 2
_THICKNESS_EYE = 2
_THICKNESS_LIPS = 2

_FACEMESH_CONTOURS_CONNECTION_STYLE = {
    face_mesh_connections.FACEMESH_LIPS:
        DrawingSpec(color=_WHITE, thickness=_THICKNESS_LIPS),
    face_mesh_connections.FACEMESH_LEFT_EYE:
        DrawingSpec(color=_GREEN, thickness=_THICKNESS_EYE),
    face_mesh_connections.FACEMESH_LEFT_EYEBROW:
        DrawingSpec(color=_BLUE, thickness=_THICKNESS_CONTOURS),
    face_mesh_connections.FACEMESH_RIGHT_EYE:
        DrawingSpec(color=_GREEN, thickness=_THICKNESS_EYE),
    face_mesh_connections.FACEMESH_RIGHT_EYEBROW:
        DrawingSpec(color=_BLUE, thickness=_THICKNESS_CONTOURS),
    face_mesh_connections.FACEMESH_FACE_OVAL:
        DrawingSpec(color=_WHITE, thickness=_THICKNESS_CONTOURS)
}

# Pose
_THICKNESS_POSE_LANDMARKS = 0
_POSE_LANDMARKS_LEFT = frozenset([
    PoseLandmark.LEFT_EYE_INNER, PoseLandmark.LEFT_EYE,
    PoseLandmark.LEFT_EYE_OUTER, PoseLandmark.LEFT_EAR, PoseLandmark.MOUTH_LEFT,
    PoseLandmark.LEFT_SHOULDER, PoseLandmark.LEFT_ELBOW,
    PoseLandmark.LEFT_WRIST, PoseLandmark.LEFT_PINKY, PoseLandmark.LEFT_INDEX,
    PoseLandmark.LEFT_THUMB, PoseLandmark.LEFT_HIP, PoseLandmark.LEFT_KNEE,
    PoseLandmark.LEFT_ANKLE, PoseLandmark.LEFT_HEEL,
    PoseLandmark.LEFT_FOOT_INDEX
])

_POSE_LANDMARKS_RIGHT = frozenset([
    PoseLandmark.RIGHT_EYE_INNER, PoseLandmark.RIGHT_EYE,
    PoseLandmark.RIGHT_EYE_OUTER, PoseLandmark.RIGHT_EAR,
    PoseLandmark.MOUTH_RIGHT, PoseLandmark.RIGHT_SHOULDER,
    PoseLandmark.RIGHT_ELBOW, PoseLandmark.RIGHT_WRIST,
    PoseLandmark.RIGHT_PINKY, PoseLandmark.RIGHT_INDEX,
    PoseLandmark.RIGHT_THUMB, PoseLandmark.RIGHT_HIP, PoseLandmark.RIGHT_KNEE,
    PoseLandmark.RIGHT_ANKLE, PoseLandmark.RIGHT_HEEL,
    PoseLandmark.RIGHT_FOOT_INDEX
])


def get_default_hand_landmarks_style() -> Mapping[int, DrawingSpec]:
  """Returns the default hand landmarks drawing style.

  Returns:
      A mapping from each hand landmark to its default drawing spec.
  """
  hand_landmark_style = {}
  for k, v in _HAND_LANDMARK_STYLE.items():
    for landmark in k:
      hand_landmark_style[landmark] = v
  return hand_landmark_style


def get_default_hand_connections_style(
) -> Mapping[Tuple[int, int], DrawingSpec]:
  """Returns the default hand connections drawing style.

  Returns:
      A mapping from each hand connection to its default drawing spec.
  """
  hand_connection_style = {}
  for k, v in _HAND_CONNECTION_STYLE.items():
    for connection in k:
      hand_connection_style[connection] = v
  return hand_connection_style


def get_default_hand_connections_style_idx(idx) -> Mapping[Tuple[int, int], DrawingSpec]:
  """Returns the default hand connections drawing style.

  Returns:
      A mapping from each hand connection to its default drawing spec.
  """
  
  HCS = _HAND_CONNECTION_STYLE_ARRAYS[idx]
  
  hand_connection_style = {}
  for k, v in HCS.items():
    for connection in k:
      hand_connection_style[connection] = v
  return hand_connection_style




def get_default_face_mesh_contours_style(
) -> Mapping[Tuple[int, int], DrawingSpec]:
  """Returns the default face mesh contours drawing style.

  Returns:
      A mapping from each face mesh contours connection to its default drawing
      spec.
  """
  face_mesh_contours_connection_style = {}
  for k, v in _FACEMESH_CONTOURS_CONNECTION_STYLE.items():
    for connection in k:
      face_mesh_contours_connection_style[connection] = v
  return face_mesh_contours_connection_style


def get_default_face_mesh_tesselation_style() -> DrawingSpec:
  """Returns the default face mesh tesselation drawing style.

  Returns:
      A DrawingSpec.
  """
  return DrawingSpec(color=_GRAY, thickness=_THICKNESS_TESSELATION)


def get_default_face_mesh_iris_connections_style(
) -> Mapping[Tuple[int, int], DrawingSpec]:
  """Returns the default face mesh iris connections drawing style.

  Returns:
       A mapping from each iris connection to its default drawing spec.
  """
  face_mesh_iris_connections_style = {}
  left_spec = DrawingSpec(color=_GREEN, thickness=_THICKNESS_CONTOURS)
  for connection in face_mesh_connections.FACEMESH_LEFT_IRIS:
    face_mesh_iris_connections_style[connection] = left_spec
  right_spec = DrawingSpec(color=_RED, thickness=_THICKNESS_CONTOURS)
  for connection in face_mesh_connections.FACEMESH_RIGHT_IRIS:
    face_mesh_iris_connections_style[connection] = right_spec
  return face_mesh_iris_connections_style


def get_default_pose_landmarks_style() -> Mapping[int, DrawingSpec]:
  """Returns the default pose landmarks drawing style.

  Returns:
      A mapping from each pose landmark to its default drawing spec.
  """
  pose_landmark_style = {}
  
  left_spec = DrawingSpec(
      color=(0, 138, 255), thickness=_THICKNESS_POSE_LANDMARKS)
  
  right_spec = DrawingSpec(
      color=(231, 217, 0), thickness=_THICKNESS_POSE_LANDMARKS)
  
  for landmark in _POSE_LANDMARKS_LEFT:
    if landmark < 11:
        pose_landmark_style[landmark] = DrawingSpec(
            color=(0, 138, 255), thickness=0)
    else:
        pose_landmark_style[landmark] = left_spec
  
  for landmark in _POSE_LANDMARKS_RIGHT:
    pose_landmark_style[landmark] = right_spec
    
  pose_landmark_style[PoseLandmark.NOSE] = DrawingSpec(
      color=_WHITE, thickness=0)
  
  return pose_landmark_style

_THICKNESS_DOT2 = 0
_RADIUS2 = 0

_HAND_LANDMARK_STYLE2 = {
    _PALM_LANMARKS:
        DrawingSpec(
            color=_RED, thickness=_THICKNESS_DOT2, circle_radius=_RADIUS2),
    _THUMP_LANDMARKS:
        DrawingSpec(
            color=_PEACH, thickness=_THICKNESS_DOT2, circle_radius=_RADIUS2),
    _INDEX_FINGER_LANDMARKS:
        DrawingSpec(
            color=_PURPLE, thickness=_THICKNESS_DOT2, circle_radius=_RADIUS2),
    _MIDDLE_FINGER_LANDMARKS:
        DrawingSpec(
            color=_YELLOW, thickness=_THICKNESS_DOT2, circle_radius=_RADIUS2),
    _RING_FINGER_LANDMARKS:
        DrawingSpec(
            color=_GREEN, thickness=_THICKNESS_DOT2, circle_radius=_RADIUS2),
    _PINKY_FINGER_LANDMARKS:
        DrawingSpec(
            color=_BLUE, thickness=_THICKNESS_DOT2, circle_radius=_RADIUS2),
}


def get_default_hand_landmarks_style2() -> Mapping[int, DrawingSpec]:
  """Returns the default hand landmarks drawing style.

  Returns:
      A mapping from each hand landmark to its default drawing spec.
  """
  hand_landmark_style = {}
  for k, v in _HAND_LANDMARK_STYLE2.items():
    for landmark in k:
      hand_landmark_style[landmark] = v
  return hand_landmark_style



def get_default_pose_connections_style(
) -> Mapping[Tuple[int, int], DrawingSpec]:
  """Returns the default hand connections drawing style.

  Returns:
      A mapping from each hand connection to its default drawing spec.
  """
  pose_connection_style = {}
  for k, v in _POSE_CONNECTION_STYLE.items():
    for connection in k:
      pose_connection_style[connection] = v
  return pose_connection_style



def get_default_pose_connections_style2(
) -> Mapping[Tuple[int, int], DrawingSpec]:
  """Returns the default hand connections drawing style.

  Returns:
      A mapping from each hand connection to its default drawing spec.
  """
  pose_connection_style = {}
  for k, v in _POSE_CONNECTION_STYLE2.items():
    for connection in k:
      pose_connection_style[connection] = v
  return pose_connection_style

