import cv2
import numpy as np
import pickle
from mhi import MHI

def imshow(img, frame_name = "monitor", ratio = 0.5):
    cv2.imshow(frame_name, cv2.resize(img, (int(img.shape[1]*ratio), int(img.shape[0]*ratio))))

def getTemplateImage(size=(270,480,3)):
    return np.zeros(size, dtype=np.uint8)

def readPickle(path):
    with open(path, 'rb') as f:
        data = pickle.load(f)
    return data


def array2landmark_hands(coordinate_list):
    with open('template_hands.pkl', 'rb') as file:
        template_landmark_list= pickle.load(file)[0]
        
    coordinate_list = coordinate_list.reshape(len(template_landmark_list.landmark),-1)

    for j,value in enumerate(template_landmark_list.landmark):
        value.x = coordinate_list[j][0]
        value.y = coordinate_list[j][1]
        value.z = coordinate_list[j][2]
            
    return template_landmark_list


def array2landmark_pose(coordinate_list):
    with open('template_pose.pkl', 'rb') as file:
        template_landmark_list= pickle.load(file)
        
    coordinate_list = coordinate_list.reshape(len(template_landmark_list.landmark),-1)

    for j,value in enumerate(template_landmark_list.landmark):
        value.x = coordinate_list[j][0]
        value.y = coordinate_list[j][1]
        value.z = coordinate_list[j][2]
        value.visibility = coordinate_list[j][3]
            
    return template_landmark_list


def array2landmark_face(coordinate_list):
    with open('template_face.pkl', 'rb') as file:
        template_landmark_list= pickle.load(file)
        
    coordinate_list = coordinate_list.reshape(len(template_landmark_list.landmark),-1)

    for j,value in enumerate(template_landmark_list.landmark):
        value.x = coordinate_list[j][0]
        value.y = coordinate_list[j][1]
        value.z = coordinate_list[j][2]
            
    return template_landmark_list


def landmark2array(value):
    fsize = len(value)
    
    for v in value:
        if v is not None:
            lsize = len(v.landmark)
            break

    
    np_landmark = np.array([[[landmark.x, landmark.y, landmark.z] for landmark in entry.landmark] 
                            if entry is not None else [[0.0, 0.0, 0.0]]*lsize 
                            for entry in value],dtype = np.float32)
    return np_landmark



def refinement(value, fun_landmark2arr, fun_arr2landmark):
    np_landmark = np.asarray([fun_landmark2arr(v) for v in value])
    state_landmark = np.array([1 if entry is not None else 0 for entry in value],dtype = np.int8)
    
    if state_landmark.sum() == 0:
        return value

    start_indices = np.where((state_landmark[:-1] == 1) & (state_landmark[1:] == 0))[0]
    finish_indices = np.where((state_landmark[:-1] == 0) & (state_landmark[1:] == 1))[0]+1

    if state_landmark[0] == 0:
        finish_indices = np.delete(finish_indices, 0)

        
    if state_landmark[-1] == 0:
        start_indices = np.delete(start_indices, -1)


    for first, last in zip(start_indices, finish_indices):
        ref_1 = np_landmark[first]
        ref_2 = np_landmark[last]
        artis = (ref_2 - ref_1) / (last-first)
        
        for k,j in enumerate(range(first,last)):
            np_landmark[j+1] = ref_1 + (k+1)*artis
            
    
    arr_np = [fun_arr2landmark(dd) for dd in np_landmark]
        
    return arr_np


def zero_fill(value, fun_landmark2arr, fun_arr2landmark):
    np_landmark = np.asarray([fun_landmark2arr(v) for v in value])
    
    value = [fun_arr2landmark(v) for v in np_landmark] 
    
    return value



def get_mhi(mask_list, im_size = (112,112)):
    img_mhi = MHI(MHI_DURATION=len(mask_list)).extract_feature_gray(mask_list)
    img_mhi = cv2.resize(img_mhi,im_size)
    return img_mhi



def split_equal(arr, split):
    
    pivot = len(arr)//split
    ll = []
    
    for i in range(1,split):
        ll.append(int(i*pivot))
    return ll



def get_mhi_multichannel(mask_list, channel_size=3, im_size = (112,112)):

    splitted_idx = split_equal(list(range(len(mask_list))),channel_size)
    channel_list = []
    pidx = 0
    for idx in splitted_idx:
        temp = MHI(MHI_DURATION=idx-pidx).extract_feature_gray(mask_list[pidx:idx])
        channel_list.append(temp)
        pidx = idx
    
    temp = MHI(MHI_DURATION=len(mask_list)-pidx).extract_feature_gray(mask_list[pidx:])
    channel_list.append(temp)
    
    img_mhi = cv2.merge(channel_list)
    
    img_mhi = cv2.resize(img_mhi,im_size)
    
    return img_mhi
    

def scale_pose(img_list, pose_data_list = [], tag_list = []):
    np_list = np.asarray(img_list)
    
    all_pose_data = []
    for pose_data,tag in zip(pose_data_list, tag_list):
        if tag == 'lhands' or tag == 'rhands':
            landmark2array =landmark2array_pose
        elif tag == 'pose':
            landmark2array =landmark2array_hands
        elif tag == 'face':
            landmark2array =landmark2array_face
    
        np_pose_data = np.asarray([landmark2array(d)[:,:3] for d in pose_data])
        all_pose_data.append(np_pose_data)
    
    np_pose_data = np.concatenate(all_pose_data,axis = 1)

    pos_kord = np.asarray(np_pose_data[:,:,:2]*(np_list.shape[2],np_list.shape[1]),dtype=np.int32)
    
    padding = 30
    
    if np_pose_data.sum()==0:
        return 0,np_list.shape[1]-1, 0, np_list.shape[2]-1
    
    min_y = max(0,np.min(pos_kord[:,:,1][pos_kord[:,:,1] > 0]) - padding)
    max_y = min(np_list.shape[1]-1,pos_kord[:,:,1].max() + padding)
    
    min_x = max(0,np.min(pos_kord[:,:,0][pos_kord[:,:,0] > 0]) - padding)
    max_x = min(np_list.shape[2]-1,pos_kord[:,:,0].max() + padding)
    
    if (max_x - min_x) > (max_y - min_y):
        piv = ((max_x - min_x) - (max_y - min_y)) // 2
        min_y = max(0, min_y - piv)
        max_y = min(np_list.shape[1] -1, max_y + piv)
    else:
        piv = ((max_y - min_y) - (max_x - min_x)) // 2
        min_x = max(0, min_x - piv)
        max_x = min(np_list.shape[2] -1, max_x + piv)
    
    return min_y,max_y,min_x,max_x

def resize_list(imglist, size = (112,112)):
    imglist = np.asarray([cv2.resize(im,size) for im in imglist])
    return imglist
    
def scale_face(img_list, pose_data):
    np_list = np.asarray(img_list)
    np_pose_data = np.asarray([landmark2array_face(d) for d in pose_data])
    pos_kord = np.asarray(np_pose_data[:,:,:2]*(np_list.shape[2],np_list.shape[1]),dtype=np.int32)
    
    padding = 20
    
    if pos_kord.sum() != 0:
        min_y = np.min(pos_kord[:,:,1][pos_kord[:,:,1] > 0]) - padding
        max_y = pos_kord[:,:,1].max() + padding
        
        min_x = np.min(pos_kord[:,:,0][pos_kord[:,:,0] > 0]) - padding
        max_x = pos_kord[:,:,0].max() + padding
    
    else:
        min_y = 0
        max_y = img_list[0].shape[0]
        min_x = 0
        max_x = img_list[0].shape[1]
    
    if (max_x - min_x) > (max_y - min_y):
        piv = ((max_x - min_x) - (max_y - min_y)) // 2
        min_y = min_y - piv
        max_y = max_y + piv
    else:
        piv = ((max_y - min_y) - (max_x - min_x)) // 2
        min_x = min_x - piv
        max_x = max_x + piv
    
    if min_x < 0: min_x=0
    
    if max_x >= (np_list.shape[2] -1): max_x=np_list.shape[2] -1
    
    if min_y < 0: min_y=0
    
    if max_y >= np_list.shape[1] -1: 
        max_y=np_list.shape[1] -1
    
    square_img = np_list[:,min_y:max_y,min_x:max_x]
        
    square_img = np.asarray([cv2.resize(im,(112,112)) for im in square_img])
    return square_img
    

def scale_hands(img_list, pose_dataL, pose_dataR):
    np_list = np.asarray(img_list)
    
    np_pose_dataL = np.asarray([landmark2array_hands(d) for d in pose_dataL])
    np_pose_dataR = np.asarray([landmark2array_hands(d) for d in pose_dataR])

    np_pose_data = np.concatenate((np_pose_dataL, np_pose_dataR),axis = 1)
    
    pos_kord = np.asarray(np_pose_data[:,:,:2]*(np_list.shape[2],np_list.shape[1]),dtype=np.int32)
    
    padding = 20
    
    if pos_kord.sum() != 0:
        min_y = np.min(pos_kord[:,:,1][pos_kord[:,:,1] > 0]) - padding
        max_y = pos_kord[:,:,1].max() + padding
        
        min_x = np.min(pos_kord[:,:,0][pos_kord[:,:,0] > 0]) - padding
        max_x = pos_kord[:,:,0].max() + padding
        
    else:
        min_y = 0
        max_y = img_list[0].shape[0]
        min_x = 0
        max_x = img_list[0].shape[1]
    
    if (max_x - min_x) > (max_y - min_y):
        piv = ((max_x - min_x) - (max_y - min_y)) // 2
        min_y = min_y - piv
        max_y = max_y + piv
    else:
        piv = ((max_y - min_y) - (max_x - min_x)) // 2
        min_x = min_x - piv
        max_x = max_x + piv
    
    if min_x < 0: min_x=0
    
    if max_x >= (np_list.shape[2] -1): max_x=np_list.shape[2] -1
    
    if min_y < 0: min_y=0
    
    if max_y >= np_list.shape[1] -1: 
        max_y=np_list.shape[1] -1
    
    square_img = np_list[:,min_y:max_y,min_x:max_x]
        
    square_img = np.asarray([cv2.resize(im,(112,112)) for im in square_img])
    return square_img

def crop_hands(landmark, img):
    pos_kord = landmark2array_hands(landmark)
    pos_kord = np.asarray(pos_kord[:,:2]*(img.shape[1],img.shape[0]),dtype=np.int32)
    
    padding = 10
    
    min_y = pos_kord[:,1].min() - padding
    max_y = pos_kord[:,1].max() + padding
    min_x = pos_kord[:,0].min() - padding
    max_x = pos_kord[:,0].max() + padding
    
    if min_x < 0: min_x=0
    
    if max_x >= (img.shape[1] -1): max_x=img.shape[1] -1
    
    if min_y < 0: min_y=0
    
    if max_y >= img.shape[0] -1: 
        max_y=img.shape[0] -1
        
    return cv2.cvtColor(img[min_y:max_y,min_x:max_x], cv2.COLOR_BGR2RGB)


def arangement_hands_data(video_dict, im_size = 56):
                         
    hand_shape = np.array([img.shape for img in video_dict]).max()
    
    res =  np.array([cv2.resize(cv2.copyMakeBorder(img,(hand_shape-img.shape[0])//2,
                                                    (hand_shape-img.shape[0])//2,
                                                    (hand_shape-img.shape[1])//2,
                                                    (hand_shape-img.shape[1])//2,
                                                    cv2.BORDER_CONSTANT,value=(0,0,0)), (im_size,im_size)) for img in video_dict])

    return res


def toGray(im):
    return cv2.cvtColor(im, cv2.COLOR_BGR2GRAY)


def get_mhi_finger(mask_list, im_size =(112,112)):

    mhi_list = []
    for mask in mask_list:
        mhi_list.append(MHI(MHI_DURATION=len(mask)).extract_feature_gray(mask))
    
    mhi_list = np.asarray(mhi_list)
    square_img = np.asarray([cv2.resize(im,im_size) for im in mhi_list])
    
    return square_img

def landmark2array_hands(landmarks):
    if landmarks is None:
        with open('template_hands.pkl', 'rb') as file:
            template_hands = pickle.load(file)[0]
            landmarks = template_hands
            
    temp = landmarks.landmark
    landmark_list = np.asarray([[v.x, v.y, v.z] for v in temp])
    return landmark_list

def landmark2array_pose(landmarks):
    if landmarks is None:
        with open('template_pose.pkl', 'rb') as file:
            template_hands = pickle.load(file)
            landmarks = template_hands
            
    temp = landmarks.landmark
    landmark_list = np.asarray([[v.x, v.y, v.z, v.visibility] for v in temp])
    return landmark_list

def landmark2array_face(landmarks):
    if landmarks is None:
        with open('template_face.pkl', 'rb') as file:
            template_hands = pickle.load(file)
            landmarks = template_hands
            
    temp = landmarks.landmark
    landmark_list = np.asarray([[v.x, v.y, v.z] for v in temp])
    return landmark_list
